package com.kyub.idea.plugin.mantis.gui.consolle;

import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.actionSystem.DataContext;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.fileEditor.FileEditorManager;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.application.Application;
import com.intellij.openapi.application.WriteAction;
import com.intellij.openapi.application.Result;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.command.CommandProcessor;
import com.intellij.psi.*;
import com.intellij.psi.javadoc.PsiDocComment;
import com.intellij.ide.DataManager;
import com.intellij.util.IncorrectOperationException;
import com.kyub.idea.plugin.mantis.gui.config.ConfigForm;
import com.kyub.idea.plugin.mantis.gui.AlertUtil;
import com.kyub.idea.plugin.mantis.gui.labels.Messages;
import com.kyub.idea.plugin.mantis.gui.labels.LabelManager;
import org.mantisbt.connect.service.IssueData;
import sun.misc.MessageUtils;

/**
 * Created by IntelliJ IDEA.
 * User: Administrator
 * Date: 20-set-2006
 * Time: 19.00.25
 * To change this template use File | Settings | File Templates.
 */
public final class IssueCodeInserter {
    private static final Logger LOG = Logger.getInstance(IssueCodeInserter.class.getName());
    private Project project;


    public IssueCodeInserter(Project project) {
        this.project = project;
    }

    public final void insertComment(IssueData data) {

        LOG.info("Inserting...");


        Application application = ApplicationManager.getApplication();
        application.runWriteAction(new Runner(project, buildText(data)));


    }

    private String buildText(IssueData data) {
        String s = "/* Introduce to resolve Issue: ";
        s = s + data.getId() + " : " + data.getDescription() + "\n */";

       LOG.info(s);  
        return s;
    }


}

final class Runner implements Runnable {
    private Project project;
    private final String text;


    public Runner(Project project, String text) {
        this.project = project;
        this.text = text;
    }

    public final void run() {


        CommandProcessor c = CommandProcessor.getInstance();
        c.executeCommand(project, new Runnable() {

            public void run() {
                Editor editor = FileEditorManager.getInstance(project).getSelectedTextEditor();


                PsiFile psiFile = PsiDocumentManager.getInstance(project).getPsiFile(editor.getDocument());


                if (editor != null && psiFile != null) {
                    PsiElement psiElement = psiFile.findElementAt(editor.getCaretModel().getOffset());
                    System.out.println("psiElement = " + psiElement);
                    PsiElementFactory factory = psiElement.getManager().getElementFactory();
                    try {
                        PsiComment newComment = factory.createCommentFromText(text, null);


                        //  psiElement.addBefore(newComment,psiElement.getFirstChild());

                        /*  System.out.println("psiElement = " + psiElement.getFirstChild());
                        System.out.println("psiElement = " + psiElement.getChildren());
                        */
                        System.out.println("psiElement Parent = " + psiElement.getParent());
                        System.out.println("psiElement Parent = " + psiElement.getParent().getFirstChild());
                        psiElement.getParent().addBefore(newComment, psiElement.getParent().getFirstChild());


                    } catch (Exception e) {
                        e.printStackTrace();
                        AlertUtil.displayErrorMessage(LabelManager.getMessageText(Messages.ISSUE_DATA_ADD_ERROR));
                    }
                }
            }
        }, "Add code comment for issue ",text.hashCode());


    }
}
