package com.kyub.idea.plugin.mantis.service;

import org.mantisbt.connect.ISession;
import org.mantisbt.connect.SessionFactory;
import org.mantisbt.connect.JMTException;
import org.mantisbt.connect.Enumeration;
import org.mantisbt.connect.service.*;


import java.net.URL;
import java.net.MalformedURLException;
import java.math.BigInteger;
import java.util.Calendar;

import com.intellij.openapi.diagnostic.Logger;
import com.kyub.idea.plugin.mantis.gui.AlertUtil;
import com.kyub.idea.plugin.mantis.gui.labels.LabelManager;
import com.kyub.idea.plugin.mantis.gui.labels.Messages;

/**
 * Created by IntelliJ IDEA.
 * User: Administrator
 * Date: 19-set-2006
 * Time: 14.55.33
 * To change this template use File | Settings | File Templates.
 */
public final class ServiceManager {

    private ISession session;

    private Long defaultProjectId;
    private ObjectRef[] issueStatus;
    private static ServiceManager instance;
    private static Logger LOG = Logger.getInstance(ServiceManager.class.getName());


    private ServiceManager(ISession session) throws JMTException {
        this.session = session;

        issueStatus = this.session.getEnum(Enumeration.STATUS);

    }


    public static ServiceManager getInstance() throws Exception {
        if(instance != null)
            return instance;
        else throw new Exception("Manager non init");
    }

    public static ServiceManager init(String url, String user, String password,Long defaultPrjId){
        instance = init(url, user, password);
        if(instance != null)
             instance.defaultProjectId = defaultPrjId;
        return instance;

    }

    public static ServiceManager init(String url, String user, String password){

        //leggi endpoint e credenziali da config



        try {
            ISession session = SessionFactory.getInstance().newSession(new URL(url),user,password);
            session.getEnum(Enumeration.STATUS);
            instance = new ServiceManager(session);
              return instance;
        } catch (JMTException e) {
             LOG.error(e);
            AlertUtil.displayErrorMessage(LabelManager.getMessageText(Messages.SERVICE_ERROR));
        } catch (MalformedURLException e) {
            AlertUtil.displayErrorMessage(LabelManager.getMessageText(Messages.SERVICE_URL_INVALID));
              LOG.error(e);

        }

         return null;


    }

    public final IssueData[] getIssueData(FilterData filterData, ProjectData project){
        try {
            return session.getIssues(project.getId().longValue(),filterData.getId().longValue());
        } catch (JMTException e) {
           LOG.error(e);
           AlertUtil.displayErrorMessage(LabelManager.getMessageText(Messages.SERVICE_ERROR));
            return null;
        }
    }
    public final IssueData[] getIssueData(FilterData filterData){
        try {
            return session.getIssues(defaultProjectId,filterData.getId().longValue());
        } catch (JMTException e) {
          LOG.error(e);
             AlertUtil.displayErrorMessage(LabelManager.getMessageText(Messages.SERVICE_ERROR));
            return null;
        }
    }
    public final FilterData[] getFilters(ProjectData project){

        try {
            return session.getFilters(project.getId().longValue());
        } catch (JMTException e) {
            LOG.error(e);
                       AlertUtil.displayErrorMessage(LabelManager.getMessageText(Messages.SERVICE_ERROR));
                        return null;
        }
    }

      public final FilterData[] getFilters(){

        try {
            return session.getFilters(defaultProjectId);
        } catch (JMTException e) {
            LOG.error(e);
                       AlertUtil.displayErrorMessage(LabelManager.getMessageText(Messages.SERVICE_ERROR));
                    return null;
        }
    }

    public final ProjectData[] getProjects(){
         try {
        return session.getAccessibleProjects();
        } catch (JMTException e) {
             LOG.error(e);
                        AlertUtil.displayErrorMessage(LabelManager.getMessageText(Messages.SERVICE_ERROR));
                    return null;
        }

    }

    public final void solveIssue(IssueData issueData){

        try {
             ObjectRef issueStat = null;
            for (int i = 0; i < issueStatus.length; i++) {
                issueStat = issueStatus[i];
                System.out.println("issueStat.getName() = " + issueStat.getName());
                if(issueStat.getName().equals("resolved"));
                //todo
            }
            if(issueStat != null){
            issueData.setStatus(issueStat);
            //WORKAROUND: mantis connect create new notes if you update a note
            issueData.setNotes(new IssueNoteData[0]);
            session.updateIssue(issueData);

            }
        } catch (JMTException e) {
            LOG.error(e);
                       AlertUtil.displayErrorMessage(LabelManager.getMessageText(Messages.SERVICE_ERROR));
                          }
        

    }

    public final void addComment(IssueData issueData,String text){

        try {
            ObjectRef visibility = session.getEnum(Enumeration.VIEW_STATES)[0];
            
            IssueNoteData noteData = new IssueNoteData(new BigInteger("0"),issueData.getHandler(),text,visibility, Calendar.getInstance(),Calendar.getInstance());
            session.addNote(issueData.getId().longValue(),noteData );
        } catch (JMTException e) {
            LOG.error(e);
            AlertUtil.displayErrorMessage(LabelManager.getMessageText(Messages.SERVICE_ERROR));

        }
    }




}
